(*
    This file is part of the Arith project -
    which provides some useful arithmetic signatures.
    
    Copyright (C) 2000 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

signature ADD_SCALE_MULT_ARITH = (* Addition, scaling and multiplication arithmetic *)
  sig
    type t
    type scalar

    (* When zeroValue is added to t, the result is t *)
    val zeroValue       : t
    val add             : (t * t) -> t
    val subtract        : (t * t) -> t

    (* Inverse number for add *)
    val negate          : t -> t

    (* Comparison *)
    val equal           : (t * t) -> bool (* The method is also called equal in the Real structure *)

    (* Scaling *)
    val scale        : (t * scalar) -> t
    val inverseScale : (t * scalar) -> t

    (* It's called unitValue and not unit to avoid conflicts with
       the SML type unit...*)
    (* When unitValue is multiplied by t, the result is t *)
    val unitValue       : t
    val multiply        : (t * t) -> t
    val divide          : (t * t) -> t

    (* Inverse number for multiply *)
    val invert          : t -> t
  end
